<?php

// ==============================================================================
// Fuction that queries Password RBL API
// Version: 1.60
// - Parameters
//    $clrpwd = Cleartext password that will be hashed and submitted to API
//    $trkid  = Optional trackingID for tracking API queries for later reporting
//    $hashtype = Default is PBKDF2, but SHA256 is also available for compatibility
// - Return values
//    0 indicates password does not exist in database
//    1 indicates password does exist in database
//      negative values indicate error  
// - Requres: PHP cURL 
// ==============================================================================
function passwordrbl_query($clrpwd="", $trkid=null, $hashtype="pbkdf2") {
  
   // You *MUST* use this salt value - do not change - see FAQ for details
   $salt1 = fe21a0daadda8301bf69a452963a2747a6c8aab4c016d9506a9af46b5f73a9ca; 

   // Create salted password hash
   if ($hashtype == "pbkdf2") {
      // NOTE: MUST use 30,000 rounds of hashing and 40 hex characters of output
      if (version_compare(PHP_VERSION, '5.5.0') >= 0) {
          // PBKDF2 hashing is built-in to PHP 5.5.0 and higher
          $hashedpwd = hash_pbkdf2('sha1', $clrpwd, $salt1, 30000, 40);
      }
      else {
         // Use below provided PBKDF2 function for older PHP versions
         // length of result is 20 bytes, returned as 40 hex characters
         $hashedpwd = pbkdf2('sha1', $clrpwd, $salt1, 30000, 20 );
      }
   }
   else {
      $hashedpwd = hash("sha256", $salt1 . $clrpwd);
   }

   // Construct URL for request to web API
   $base_url  = 'https://api.passwordrbl.com/query.php';
   $url       = $base_url . '?hashvalue=' . $hashedpwd; 
   if (trkid != null) { $url = $url . '&trackingid=' . $trkid; }
   
   // Set cURL options and execute the API call via HTTPS GET request
   $curl = curl_init();                                //The default curl method is GET
     curl_setopt($curl, CURLOPT_URL, $url);            //set the URL
     curl_setopt($curl, CURLOPT_RETURNTRANSFER, true); //have cURL return the API response as a string value
     curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 5);    //set the maximum time to wait for API Response
     $curl_response = curl_exec($curl);                //execute the HTTP request
     $curl_errval = curl_error($curl);                 //Check for errors returned from cURL
   curl_close($curl);                                  //Close the cURL session to the API to free up resources

   // If cURL had errors, return -1
   if ($curl_errval != "") {
      // uncomment the following while debugging 
      // echo "cURL had errors<BR> cURL response was: ", $curl_response,"<BR>cURL error text: ", $curl_errval, "<BR>";
      return -1;
   }//if

   return $curl_response;                              //return the response from the API
}//function




/*
 * PBKDF2 key derivation function as defined by RSA's PKCS #5: https://www.ietf.org/rfc/rfc2898.txt
 * $algorithm - The hash algorithm to use. Recommended: SHA256
 * $password - The password.
 * $salt - A salt that is unique to the password.
 * $count - Iteration count. Higher is better, but slower. Recommended: At least 1000.
 * $key_length - The length of the derived key in bytes.
 * $raw_output - If true, the key is returned in raw binary format. Hex encoded otherwise.
 * Returns: A $key_length-byte key derived from the password and salt.
 * Test vectors can be found here: https://www.ietf.org/rfc/rfc6070.txt
 * Originally created by defuse.ca, improvements by variations-of-shadow.com
 */
function pbkdf2($algorithm, $password, $salt, $count, $key_length, $raw_output = false)
{
    $algorithm = strtolower($algorithm);
    if(!in_array($algorithm, hash_algos(), true))
        trigger_error('PBKDF2 ERROR: Invalid hash algorithm.', E_USER_ERROR);
    if($count <= 0 || $key_length <= 0)
        trigger_error('PBKDF2 ERROR: Invalid parameters.', E_USER_ERROR);

    if (function_exists("hash_pbkdf2")) {
        // The output length is in NIBBLES (4-bits) if $raw_output is false!
        if (!$raw_output) {
            $key_length = $key_length * 2;
        }
        return hash_pbkdf2($algorithm, $password, $salt, $count, $key_length, $raw_output);
    }

    $hash_length = strlen(hash($algorithm, "", true));
    $block_count = ceil($key_length / $hash_length);

    $output = "";
    for($i = 1; $i <= $block_count; $i++) {
        // $i encoded as 4 bytes, big endian.
        $last = $salt . pack("N", $i);
        // first iteration
        $last = $xorsum = hash_hmac($algorithm, $last, $password, true);
        // perform the other $count - 1 iterations
        for ($j = 1; $j < $count; $j++) {
            $xorsum ^= ($last = hash_hmac($algorithm, $last, $password, true));
        }
        $output .= $xorsum;
    }

    if($raw_output)
        return substr($output, 0, $key_length);
    else
        return bin2hex(substr($output, 0, $key_length));
}

?>
